DROP TABLE IF EXISTS `Form`;
CREATE TABLE IF NOT EXISTS `Form` (
  `id`                       INT(11)                                                   NOT NULL  AUTO_INCREMENT,
  `name`                     VARCHAR(255)                                              NOT NULL  DEFAULT '',
  `title`                    VARCHAR(255)                                              NOT NULL  DEFAULT '',
  `noteInternal`             TEXT                                                      NOT NULL,
  `tableName`                VARCHAR(255)                                              NOT NULL  DEFAULT '',


  `permitNew`                ENUM('sip', 'logged_in', 'logged_out', 'always', 'never') NOT NULL  DEFAULT 'sip',
  `permitEdit`               ENUM('sip', 'logged_in', 'logged_out', 'always', 'never') NOT NULL  DEFAULT 'sip',
  `render`                   ENUM('plain', 'table', 'bootstrap')                       NOT NULL  DEFAULT 'plain',
  `showButton`               SET('new', 'delete', 'close', 'save')                     NOT NULL  DEFAULT 'new,delete,close,save',
  `multiMode`                ENUM('none', 'horizontal', 'vertical')                    NOT NULL  DEFAULT 'none',
  `multiSql`                 TEXT                                                      NOT NULL,
  `multiDetailForm`          VARCHAR(255)                                              NOT NULL  DEFAULT '',
  `multiDetailFormParameter` VARCHAR(255)                                              NOT NULL  DEFAULT '',

  `forwardMode`              ENUM('client', 'no', 'page')                              NOT NULL  DEFAULT 'client',
  `forwardPage`              VARCHAR(255)                                              NOT NULL  DEFAULT '',

  `bsLabelColumns`           VARCHAR(255)                                              NOT NULL  DEFAULT '',
  `bsInputColumns`           VARCHAR(255)                                              NOT NULL  DEFAULT '',
  `bsNoteColumns`            VARCHAR(255)                                              NOT NULL  DEFAULT '',

  `parameter`                TEXT                                                      NOT NULL,

  `deleted`                  ENUM('yes', 'no')                                         NOT NULL  DEFAULT 'no',
  `modified`                 TIMESTAMP                                                 NOT NULL  DEFAULT CURRENT_TIMESTAMP,
  `created`                  DATETIME                                                  NOT NULL  DEFAULT '0000-00-00 00:00:00',

  PRIMARY KEY (`id`),
  KEY `name` (`name`),
  KEY `name_deleted` (`name`, `deleted`)
)
  ENGINE = InnoDB
  DEFAULT CHARSET = utf8
  AUTO_INCREMENT = 0;

#--
#-- Triggers `Form`
#--
#DROP TRIGGER IF EXISTS `on_Form_update_modified`;
#DELIMITER //
#CREATE TRIGGER `on_Form_update_modified` BEFORE UPDATE ON `Form`
#FOR EACH ROW SET NEW.modified =
#current_timestamp()
#//
#DELIMITER ;

# ----------------------------------------
# FormElement

DROP TABLE IF EXISTS `FormElement`;
CREATE TABLE IF NOT EXISTS `FormElement` (
  `id`              INT(11)                                                                       NOT NULL  AUTO_INCREMENT,
  `formId`          INT(11)                                                                       NOT NULL,
  `feIdContainer`   INT(11)                                                                       NOT NULL  DEFAULT '0',
  `dynamicUpdate`   ENUM('yes', 'no')                                                             NOT NULL  DEFAULT 'no',

  `enabled`         ENUM('yes', 'no')                                                             NOT NULL  DEFAULT 'yes',

  `name`            VARCHAR(255)                                                                  NOT NULL  DEFAULT '',
  `label`           VARCHAR(255)                                                                  NOT NULL  DEFAULT '',

  `mode`            ENUM('show', 'readonly', 'required', 'lock', 'disabled')                      NOT NULL  DEFAULT 'show',
  `class`           ENUM('native', 'action', 'container')                                         NOT NULL  DEFAULT 'native',
  `type`            ENUM('checkbox', 'dateJQW', 'datetimeJQW', 'gridJQW', 'hidden', 'text',
                         'note', 'password', 'radio', 'select', 'subrecord', 'upload', 'fieldset', 'pill',
                         'before_load', 'before_save', 'before_insert', 'before_update', 'before_delete', 'after_load',
                         'after_save', 'after_insert', 'after_update', 'after_delete', 'feGroup',
                         'sendmail')                                                              NOT NULL  DEFAULT 'text',
  `subrecordOption` SET('edit', 'delete', 'new')                                                  NOT NULL  DEFAULT '',
  `checkType`       ENUM('alnumx', 'digit', 'email', 'min|max', 'min|max date', 'pattern', 'all') NOT NULL  DEFAULT 'alnumx',
  `checkPattern`    VARCHAR(255)                                                                  NOT NULL  DEFAULT '',

  `onChange`        VARCHAR(255)                                                                  NOT NULL  DEFAULT '',

  `ord`             INT(11)                                                                       NOT NULL  DEFAULT '0',
  `tabindex`        INT(11)                                                                       NOT NULL  DEFAULT '0',

  `size`            VARCHAR(255)                                                                  NOT NULL  DEFAULT '',
  `maxLength`       VARCHAR(255)                                                                  NOT NULL  DEFAULT '',
  `note`            TEXT                                                                          NOT NULL,
  `tooltip`         VARCHAR(255)                                                                  NOT NULL  DEFAULT '',
  `placeholder`     VARCHAR(255)                                                                  NOT NULL  DEFAULT '',

  `value`           TEXT                                                                          NOT NULL,
  `sql1`            TEXT                                                                          NOT NULL,
  `sql2`            TEXT                                                                          NOT NULL,
  `parameter`       TEXT                                                                          NOT NULL,
  `clientJs`        TEXT                                                                          NOT NULL,

  `feGroup`         VARCHAR(255)                                                                  NOT NULL  DEFAULT '',
  `deleted`         ENUM('yes', 'no')                                                             NOT NULL  DEFAULT 'no',
  `modified`        TIMESTAMP                                                                     NOT NULL  DEFAULT CURRENT_TIMESTAMP,
  `created`         DATETIME                                                                      NOT NULL  DEFAULT '0000-00-00 00:00:00',

  PRIMARY KEY (`id`),
  KEY `formId` (`formId`),
  KEY `formId_class_enabled_deleted` (`formId`, `class`, `enabled`, `deleted`)
)
  ENGINE = InnoDB
  DEFAULT CHARSET = utf8
  AUTO_INCREMENT = 0;

#--
#-- Triggers `FormElement`
#--
#DROP TRIGGER IF EXISTS `on_FormEelement_update_modified`;
#DELIMITER //
#CREATE TRIGGER `on_FormEelement_update_modified` BEFORE UPDATE ON `FormElement`
#FOR EACH ROW SET NEW.modified =
#current_timestamp()
#//
#DELIMITER ;


#
# FormEditor: Form
INSERT INTO Form (name, title, noteInternal, tableName, permitNew, permitEdit, render, multiSql, parameter) VALUES
  ('form', 'Form Editor: {{SELECT id, " / ", name FROM Form WHERE id = {{r:S0}}}}', 'Please secure the form',
   'Form', 'always', 'always', 'bootstrap', '', 'maxVisiblePill=5\nclass=container-fluid');

# FormEditor: FormElements
INSERT INTO FormElement (formId, name, label, mode, type, checkType, class, ord, size, maxLength, note, clientJs, value, sql1, sql2, parameter, feIdContainer, subrecordOption)
VALUES
  (1, 'basic', 'Basic', 'show', 'pill', 'all', 'container', 10, 0, 0, '', '', '', '', '', '', 0, ''),
  (1, 'permission', 'Permission', 'show', 'pill', 'all', 'container', 20, 0, 0, '', '', '', '', '', '', 0, ''),
  (1, 'various', 'Various', 'show', 'pill', 'all', 'container', 30, 0, 0, '', '', '', '', '', '', 0, ''),
  (1, 'multi', 'Multi', 'show', 'pill', 'all', 'container', 40, 0, 0, '', '', '', '', '', '', 0, ''),
  (1, 'formelement', 'Formelement', 'show', 'pill', 'all', 'container', 50, 0, 0, '', '', '', '', '', '', 0, ''),

  (1, 'id', 'id', 'readonly', 'text', 'all', 'native', 100, 10, 11, '', '', '', '', '', '', 1, ''),
  (1, 'name', 'Name', 'required', 'text', 'all', 'native', 120, 40, 255, '', '', '', '', '', 'autofocus', 1, ''),
  (1, 'title', 'Title', 'show', 'text', 'all', 'native', 130, 40, 255, '', '', '', '', '', '', 1, ''),
  (1, 'noteInternal', 'Note', 'show', 'text', 'all', 'native', 140, '40,3', 0, '', '', '', '', '', '', 1, ''),
  (1, 'tableName', 'Table', 'required', 'select', 'all', 'native', 150, 0, 0, '', '', '', '{{!SHOW tables}}', '', 'emptyItemAtStart', 1, ''),

  (1, 'permitNew', 'Permit New', 'show', 'radio', 'all', 'native', 160, 0, 0, '', '', '', '', '', '', 2, ''),
  (1, 'permitEdit', 'Permit Edit', 'show', 'radio', 'all', 'native', 170, 0, 0, '', '', '', '', '', '', 2, ''),
  (1, 'render', 'Render', 'show', 'radio', 'all', 'native', 190, 0, 0, '', '', '', '', '', '', 2, ''),
  (1, 'showButton', 'Show button', 'show', 'checkbox', 'all', 'native', 200, 0, 0, '', '', '', '', '', 'checkBoxMode = multi\norientation=vertical', 2, ''),

  (1, 'forwardMode', 'Forward', 'show', 'radio', 'all', 'native', 260, 0, 0, '', '', '', '', '', '', 3, ''),
  (1, 'forwardPage', 'Forward Page', 'show', 'text', 'all', 'native', 270, 40, 255, '', '', '', '', '', '', 3, ''),
  (1, 'parameter', 'Parameter', 'show', 'text', 'all', 'native', 275, '40,3', 0, '', '', '', '', '', '', 3, ''),

  (1, 'bsLabelColumns', 'BS Label Columns', 'show', 'text', 'all', 'native', 280, 40, 250, '', '', '', '', '', '', 3, ''),
  (1, 'bsInputColumns', 'BS Input Columns', 'show', 'text', 'all', 'native', 290, 40, 250, '', '', '', '', '', '', 3, ''),
  (1, 'bsNoteColumns', 'BS Note Columns', 'show', 'text', 'all', 'native', 300, 40, 250, '', '', '', '', '', '', 3, ''),

  (1, 'deleted', 'Deleted', 'show', 'checkbox', 'all', 'native', 405, 0, 0, '', '', '', '', '', '', 3, ''),
  (1, 'modified', 'Modified', 'readonly', 'text', 'all', 'native', 410, 40, 20, '', '', '', '', '', '', 3, ''),
  (1, 'created', 'Created', 'readonly', 'text', 'all', 'native', 420, 40, 20, '', '', '', '', '', '', 3, ''),

  (1, 'multi', 'Multi', 'show', 'fieldset', 'all', 'native', 210, 0, 0, '', '', '', '', '', '', 4, ''),
  (1, 'multiMode', 'Multi Mode', 'show', 'radio', 'all', 'native', 220, 0, 0, '', '', '', '', '', '', 4, ''),
  (1, 'multiSql', 'Multi SQL', 'show', 'text', 'all', 'native', 230, '40,3', 0, '', '', '', '', '', '', 4, ''),
  (1, 'multiDetailForm', 'Multi Detail Form', 'show', 'text', 'all', 'native', 240, 40, 255, '', '', '', '', '', '', 4,
   ''),
  (1, 'multiDetailFormParameter', 'Multi Detail Form Parameter', 'show', 'text', 'all', 'native', 250, 40, 255, '', '',
   '', '',
   '', '', 4, ''),

  (1, '', 'FormElements', 'show', 'subrecord', 'all', 'native', 500, 0, 0, '', '', '',
   '{{!SELECT id, feIdContainer,	name,	label,	mode,	class,	type, ord, size, sql1, parameter FROM FormElement WHERE formId={{id:R0}} ORDER BY ord, id}}',
   '', 'form=formElement\ndetail=id:formId', 5, 'new,edit,delete');

#
# FormEditor: FormElement
INSERT INTO Form (name, title, noteInternal, tableName, permitNew, permitEdit, render, multiSql, parameter) VALUES
  ('formElement',
   'Form Element Editor. Form : {{SELECT f.id, " / ",  f.name  FROM FormElement AS fe, Form AS f WHERE fe.id = {{r:S0}} AND fe.formId=f.id }}',
   'Please secure the form',
   'FormElement', 'always', 'always', 'bootstrap', '', 'maxVisiblePill=5');

# FormEditor: FormElements
INSERT INTO FormElement (id, formId, name, label, mode, type, checkType, class, ord, size, maxLength, note, clientJs, value, sql1, sql2, parameter, feIdContainer, subrecordOption)
VALUES

  (100, 2, 'basic', 'Basic', 'show', 'pill', 'all', 'container', 10, 0, 0, '', '', '', '', '', '', 0, ''),
  (101, 2, 'check_order', 'Check & Order', 'show', 'pill', 'all', 'container', 20, 0, 0, '', '', '', '', '', '', 0, ''),
  (102, 2, 'layout', 'Layout', 'show', 'pill', 'all', 'container', 20, 0, 0, '', '', '', '', '', '', 0, ''),
  (103, 2, 'value', 'Value', 'show', 'pill', 'all', 'container', 20, 0, 0, '', '', '', '', '', '', 0, ''),
  (104, 2, 'info', 'Info', 'show', 'pill', 'all', 'container', 20, 0, 0, '', '', '', '', '', '', 0, '');

INSERT INTO FormElement (formId, name, label, mode, type, checkType, class, ord, size, maxLength, note, clientJs, value, sql1, sql2, parameter, feIdContainer, subrecordOption, dynamicUpdate)
VALUES

  (2, 'id', 'id', 'readonly', 'text', 'all', 'native', 100, 10, 11, '', '', '', '', '', '', 100, '', 'no'),
  (2, 'formId', 'formId', 'readonly', 'text', 'all', 'native', 110, 40, 255, '', '', '', '', '', '', 100, '', 'no'),
  (2, 'feIdContainer', 'Container', 'show', 'select', 'all', 'native', 120, 0, 0, '', '', '',
   '{{!SELECT fe.id, CONCAT(fe.class, " / ", fe.label) FROM FormElement As fe WHERE fe.formId={{formId}} AND fe.class="container" ORDER BY fe.ord }}',
   '', 'emptyItemAtStart', 100, '', 'no'),
  (2, 'enabled', 'Enabled', 'show', 'checkbox', 'all', 'native', 130, 0, 0, '', '', '', '', '', '', 100, '', 'no'),
  (2, 'dynamicUpdate', 'Dynamic Update', 'show', 'checkbox', 'all', 'native', 135, 0, 0, 'This element will be updated on change and trigger other.', '', '', '', '', '', 100, '', 'no'),
  (2, 'name', 'Name', 'required', 'text', 'all', 'native', 140, 40, 255, '', '', '', '', '', '', 100, '', 'no'),
  (2, 'label', 'Label', 'show', 'text', 'all', 'native', 150, 40, 255, '', '', '', '', '', '', 100, '', 'no'),
  (2, 'mode', 'Mode', 'show', 'select', 'all', 'native', 160, 0, 255, '', '', '', '', '', '', 100, '', 'no'),
  (2, 'class', 'Class', 'show', 'select', 'all', 'native', 170, 0, 255, '', '', '{{class:FSRD0:alnumx}}', '', '', '', 100, '', 'yes'),
  (2, 'type', 'Type', 'show', 'select', 'all', 'native', 180, 0, 255, '', '', '', '', '',
   'itemList={{SELECT IF( "{{class:FRD0:alnumx}}"="native","checkbox,dateJQW,datetimeJQW,gridJQW,hidden,text,note,password,radio,select,subrecord,upload", IF("{{class:FRD0:alnumx}}"="action","before_load,before_save,before_insert,before_update,before_delete,after_load,after_save,after_insert,after_update,after_delete,feGroup,sendmail", "fieldset,pill")  ) }}',
   100, '', 'yes'),
  (2, 'subrecordOption', 'Subrecord Option', 'show', 'checkbox', 'all', 'native', 190, 0, 0, '', '', '', '', '', '', 100, '', 'no'),
  (2, 'checkType', 'Check Type', 'show', 'select', 'all', 'native', 200, 0, 255, '', '', '', '', '', '', 101, '', 'no'),
  (2, 'checkPattern', 'Check Pattern', 'show', 'text', 'all', 'native', 210, 40, 255, '', '', '', '', '', '', 101, '', 'no'),
  (2, 'onChange', 'JS onChange', 'show', 'text', 'all', 'native', 220, 40, 255, '', '', '', '', '', '', 101, '', 'no'),
  (2, 'ord', 'Order', 'show', 'text', 'all', 'native', 230, 40, 255, '', '', '', '', '', '', 101, '', 'no'),
  (2, 'tabindex', 'tabindex', 'show', 'text', 'all', 'native', 240, 40, 255, '', '', '', '', '', '', 101, '', 'no'),
  (2, 'size', 'Size', 'show', 'text', 'all', 'native', 250, 40, 255, '', '', '', '', '', '', 102, '', 'no'),
  (2, 'maxLength', 'Maxlength', 'show', 'text', 'all', 'native', 260, 40, 255, '', '', '', '', '', '', 102, '', 'no'),
  (2, 'note', 'note', 'show', 'text', 'all', 'native', 270, 40, 255, '', '', '', '', '', '', 102, '', 'no'),
  (2, 'tooltip', 'Tooltip', 'show', 'text', 'all', 'native', 280, 40, 255, '', '', '', '', '', '', 102, '', 'no'),
  (2, 'placeholder', 'Placeholder', 'show', 'text', 'all', 'native', 290, 40, 255, '', '', '', '', '', '', 102, '', 'no'),
  (2, 'value', 'value', 'show', 'text', 'all', 'native', 300, 40, 255, '', '', '', '', '', '', 103, '', 'no'),
  (2, 'sql1', 'sql1', 'show', 'text', 'all', 'native', 310, '70,5', 255, '', '', '', '', '', '', 103, '', 'no'),
  (2, 'parameter', 'Parameter', 'show', 'text', 'all', 'native', 320, '40,4', 255, '', '', '', '', '', '', 103, '',
   'no'),
  (2, 'clientJs', 'ClientJS', 'show', 'text', 'all', 'native', 330, 40, 255, '', '', '', '', '', '', 103, '', 'no'),
  (2, 'feGroup', 'feGroup', 'show', 'text', 'all', 'native', 340, 40, 255, '', '', '', '', '', '', 104, '', 'no'),
  (2, 'deleted', 'Deleted', 'show', 'checkbox', 'all', 'native', 350, 0, 0, '', '', '', '', '', '', 104, '', 'no'),
  (2, 'modified', 'Modified', 'readonly', 'text', 'all', 'native', 360, 40, 20, '', '', '', '', '', '', 104, '', 'no'),
  (2, 'created', 'Created', 'readonly', 'text', 'all', 'native', 370, 40, 20, '', '', '', '', '', '', 104, '', 'no');

# FormEditor: Small
INSERT INTO Form (name, title, noteInternal, tableName, permitNew, permitEdit, render, multiSql, parameter) VALUES
  ('person', 'Person {{SELECT ": ", firstName, " ", name, " (", id, ")" FROM Person WHERE id = {{r:S0}}}}',
   'Please secure the form',
   'Person', 'always', 'always', 'bootstrap', '', '');

# FormEditor: FormElements
INSERT INTO FormElement (id, formId, name, label, mode, type, checkType, class, ord, size, maxLength, note, clientJs, value, sql1, sql2, parameter, feIdContainer, subrecordOption)
VALUES

  (200, 3, 'name', 'Name', 'show', 'text', 'all', 'native', 10, 50, 255, '', '', '', '', '', '', 0, ''),
  (201, 3, 'firstname', 'Firstname', 'show', 'text', 'all', 'native', 10, 50, 255, '', '', '', '', '', '', 0, '');

# ----------------------------------------------------------------------
#

# Form: plain
INSERT INTO Form (name, title, noteInternal, tableName, permitNew, permitEdit, render, multiSql, parameter) VALUES
  ('formplain', 'Form: Plain', '', 'Form', 'always', 'always', 'plain', '', '');

# FormEditor: FormElements
INSERT INTO FormElement (id, formId, name, label, mode, type, checkType, class, ord, size, maxLength, note, clientJs, value, sql1, sql2, parameter, feIdContainer, subrecordOption)
VALUES
  (300, 4, 'id', 'id', 'readonly', 'text', 'all', 'native', 100, 10, 11, '', '', '', '', '', '', 0, ''),
  (310, 4, 'name', 'Name', 'show', 'text', 'all', 'native', 120, 40, 255, '', '', '', '', '', '', 0, '');

# Form: table
INSERT INTO Form (name, title, noteInternal, tableName, permitNew, permitEdit, render, multiSql, parameter) VALUES
  ('formtable', 'Form: Table', '', 'Form', 'always', 'always', 'table', '', '');

# FormEditor: FormElements
INSERT INTO FormElement (id, formId, name, label, mode, type, checkType, class, ord, size, maxLength, note, clientJs, value, sql1, sql2, parameter, feIdContainer, subrecordOption)
VALUES
  (400, 5, 'id', 'id', 'readonly', 'text', 'all', 'native', 100, 10, 11, '', '', '', '', '', '', 0, ''),
  (410, 5, 'name', 'Name', 'show', 'text', 'all', 'native', 120, 40, 255, '', '', '', '', '', '', 0, '');
